@@ -1,7 +1,9 @@
 from inspect import iscoroutinefunction
+from typing import Tuple
 
 from . import constants
-from . import register
+from .typing import CT, LFMethod, function_types
+from .register import register_check_method_type
 
 
 check_callback_error_message__template = (
@@ -10,15 +12,26 @@ check_callback_error_message__template = (
 
 reversed_sync_or_async = tuple(reversed(constants.sync_or_async))
 
-check_callback_signature__error_message = (
+check_callback_signature__error_message: Tuple[str, ...] = (
     check_callback_error_message__template % constants.sync_or_async,
     check_callback_error_message__template % reversed_sync_or_async,
 )
 
 
-def check_callback_signature(reason, func):
+def check_callback_signature(reason: LFMethod, func: LFMethod) -> None:
+    """
+    When `reason` is syncronous, `func` have to be syncronous too
+    Asyncronous `reason` executors may to execute both `func` types
+    """
+
+    assert isinstance(reason, function_types)
+    assert isinstance(func, function_types)
+
     reason_async = iscoroutinefunction(reason)
-    if (reason_async == iscoroutinefunction(func)):
+    if reason_async:
+        return
+
+    if not iscoroutinefunction(func):
         return
 
     template = check_callback_signature__error_message[reason_async]
@@ -31,8 +44,8 @@ def check_callback_signature(reason, func):
     raise ValueError(error_message)
 
 
-@register.register_check_method_type("gather")
-def check_direction(target):
+@register_check_method_type("gather")
+def gather(target: CT) -> None:
     if not iscoroutinefunction(target):
         error_message = "Can not `asyncio.gather` syncronous method"
         raise ValueError(error_message, target)
